from redirector.utils.const import MAPS_DIR, CONFIG_DIR

class Generator:
    def __init__(self):
        self.map_gen = MapGenerator()
        self.conf_gen = ConfigGenerator()
    
    def get_conf_dirs(self):
        return MAPS_DIR, CONFIG_DIR

    def write_map(self, maps_dir, project_name, redirects_map, redirects_with_options):
        try:
            with open("{}/{}.map".format(maps_dir,project_name), "w+") as map_file:
                map_file.write(redirects_map)
            for code, data in redirects_with_options:
                if code == "301":
                    code = "permanent"
                elif code == "302":
                    code = "redirect"
                with open(maps_dir + "/%s_%s_options.map" % (project_name, code), "w+") as map_file:
                    map_file.write(data)
        except Exception as e:
            print(e)
            raise self.GenerationError("Can\'t generate new .map file \
                from %s .map file for %s project" % (map_file, project_name), e)

    def write_conf(self, config_dir, project_name, conf_data):
        try:
            with open(config_dir + "/%s.conf" % project_name, "w+") as conf_file:
                conf_file.write(conf_data)
        except Exception as e:
            print(e)
            raise self.GenerationError("Can\'t generate new .conf \
                from %s .conf file for %s project" % (conf_file, project_name), e)
                
    def generate(self, redirects_data, project_name, prefix):
        
        redirects_map = self.map_gen.generate_map(redirects_data[0], project_name, prefix)
        redirects_with_options = self.map_gen.generate_opt_map(redirects_data[1], project_name, prefix)
        conf_data = self.conf_gen.generate_config(project_name, [code for code, data in redirects_with_options],prefix)

        maps_dir, config_dir = self.get_conf_dirs()
        self.write_map(maps_dir, project_name, redirects_map, redirects_with_options)
        self.write_conf(config_dir, project_name, conf_data)
    
    class GenerationError(Exception):
        def __init__(self, message, errors):
            super().__init__(message)
            self.errors = errors



class MapGenerator:
    def __init__(self):
        self.start_line = "map $uri $%s_redirect {"
        self.opt_start_line = "map $uri $%s_%s_redirect {" # (project_name, redirect_option)
     
        self.endline = "\n}"

    def generate_map(self, redirects, project_name, prefix):
        if prefix:
            data = self.start_line % prefix
        else:
            data = self.start_line % project_name
        if project_name == prefix:
            prefix = "/{}".format(prefix)
        for arg1, arg2 in redirects:
            # print('redirect is', arg1, arg2)
            data += "\n\t%s\t%s;" % (prefix+arg1, prefix+arg2)
        data += self.endline
        return data
    
    def generate_opt_map(self, redirect_dict, project_name, prefix):
        result = []
        for code, redirects in redirect_dict.items():
            if project_name == prefix:
                prefix = "/{}".format(prefix)
            if code == "301":
                code = "permanent"
                data = self.opt_start_line % (project_name, code)
                for arg1, arg2 in redirects:
                    data += "\n\t%s\t%s;" % (prefix+arg1, arg2)
                data += self.endline
            elif code == "302":
                code = "redirect"
                data = self.opt_start_line % (project_name, code)
                for arg1, arg2 in redirects:
                    data += "\n\t%s\t%s;" % (prefix+arg1, prefix+arg2)
                data += self.endline
            elif code == "status=301":  
                data = self.opt_start_line % (project_name, code)
                for arg1, arg2 in redirects:
                    data += "\n\t%s\t%s;" % (prefix+arg1, prefix+arg2)
                data += self.endline
            result.append((code, data))
        return result

            
            
class ConfigGenerator:
    def __init__(self):
        self.start_line = "if ($%s_redirect) {\n"
        self.opt_start_line = "if ($%s_%s_redirect) {\n"
        self.rewrite_line = "\trewrite ^%s/(.*)$ $%s_redirect %s;\n"
        self.opt_rewrite_line = "\trewrite ^%s/(.*)$ $%s_%s_redirect %s;\n"
        self.option_line = "return %s;\n}\n"

    def generate_config(self, project_name, options, prefix):
        if project_name == prefix:
            prefix = "/{}".format(prefix)
        data = (self.start_line % project_name) + (self.rewrite_line % (prefix, project_name, "redirect")) + "}\n"
        for code in options:
            if code == "permanent":
                data += (self.opt_start_line % (project_name, "permanent")) + (self.opt_rewrite_line % (prefix, project_name, "permanent", "permanent")) + "}\n"
            elif code == "redirect":
                data += (self.opt_start_line % (project_name, "redirect")) + (self.opt_rewrite_line % (prefix, project_name, "redirect", "redirect")) + "}\n"
            else:
                data += (self.opt_start_line % (project_name, code)) + (self.opt_rewrite_line % (prefix, project_name, code,  "redirect")) + (self.option_line % code)
        return data

